#!/bin/bash
COM_ERR=1
INSTALL_DIR=/opt/temp_readout
CONF_DIR=$INSTALL_DIR/conf
CONF_FILES=$CONF_DIR/*
LOG_FILE_PATH=$INSTALL_DIR/logs
LIB=temp_readout.lib
NUM_NTC=32
NUM_THRM=4
NUM_CONFS=$(ls $CONF_DIR | wc -l)
RET=0

for f in $CONF_FILES
do
	GLOBAL_CONF=$f
	source $f
	args="-file_write_delay $FILE_SAVE_DELAY_S -save_to_file $SAVE_TO_FILE -max_file_size_mb $MAX_FILE_SIZE_MB -log_file_path $LOG_FILE_PATH -buzzer_on $BUZZER_ON -units $UNITS"
	break;
done


for f in $CONF_FILES
do
	#echo "Processing $f"
	source $f
	
	args+=" -i2c_addr $I2C_ADDR -conf_file $f -ac_filter $AC_FILTER"
	
	x=1
	while [ $x -le $NUM_NTC ]; do
		args+=" -ntc_num $x"
		
		var="$""NTC_""$x""_NAME"
		eval var=$var
		args+=" -ntc_name $var"
			
		var="$""NTC_""$x""_EN"
		eval var=$var
		args+=" -ntc_en $var"
		
		var="$""NTC_""$x""_R25"
		eval var=$var
		args+=" -ntc_r25 $var"
	
		var="$""NTC_""$x""_BETA"
		eval var=$var
		args+=" -ntc_beta $var"
		
		var="$""NTC_""$x""_ALARM_HIGH"
		eval var=$var
		args+=" -ntc_alarm_high $var"
		
		var="$""NTC_""$x""_ALARM_LOW"
		eval var=$var
		args+=" -ntc_alarm_low $var"
		
		var="$""NTC_""$x""_OFFSET"
		eval var=$var
		args+=" -ntc_offset $var"
				
		let x=x+1
	done
	
	x=1
	while [ $x -le $NUM_THRM ]; do
		args+=" -thrm_num $x"
		
		var="$""THRM_""$x""_NAME"
		eval var=$var
		args+=" -thrm_name $var"
			
		var="$""THRM_""$x""_EN"
		eval var=$var
		args+=" -thrm_en $var"
		
		var="$""THRM_""$x""_TYPE"
		eval var=$var
		args+=" -thrm_type $var"
		
		var="$""THRM_""$x""_ALARM_HIGH"
		eval var=$var
		args+=" -thrm_alarm_high $var"
		
		var="$""THRM_""$x""_ALARM_LOW"
		eval var=$var
		args+=" -thrm_alarm_low $var"
		
		var="$""THRM_""$x""_OFFSET"
		eval var=$var
		args+=" -thrm_offset $var"
				
		let x=x+1
	done
done

#echo $args
#exit 0

if [ $# -eq 0 ]
then
	echo "Please type \"lcd.sh -help\" for a list of commands"
	exit 0
fi


if [ $1 = "-help" ]
then
	COM_ERR=0
	
	echo "List of commands"
	echo ""
	
	echo "-help"
	echo "-board_info"
	echo "-start"
	echo "-config"
	echo "-read_settings"
	echo "-i2c_find"
	echo "-i2c_change_addr <new_addr>"
	echo "-sw_rev"
	echo "-read_conf"
	echo "-fw_update <file_name>"
	echo "-ntc_calc_beta <C/F> <t1> <r1> <t2> <r2>"
	echo "-init_hw"
	echo "-read_sensor <num>"
	echo "-buzzer"

#Read Board information
#If just one board and one .conf file then temp_readout.sh -board_info
#If more than one board, then  temp_readout.sh -board_info <.conf file name>
elif [ $1 = "-board_info" ]
then
	COM_ERR=0
	
	if [ $NUM_CONFS -ne 1 ] && [ $# -ne 2 ]
	then
		echo "ERROR: Expecting one argument, <.conf file name>"
		echo "For example: temp_readout.sh -board_info temp_readout_1.conf"
		echo "Listing .conf files: ls $CONF_DIR"
		ls $CONF_DIR
		exit 1
	
	else	
	
		if [ $NUM_CONFS -ne 1 ]
		then
			
			if [ -f $CONF_DIR/$2 ]
			then
				source $CONF_DIR/$2
			else
				echo "File $CONF_DIR/$2 not found"
				exit 1;
			fi
				
		else
			source $CONF_DIR/*
		fi
		
		sudo $INSTALL_DIR/$LIB -i2c_addr $I2C_ADDR -board_info 
		if [ $? -ne 0 ]
		then
			echo "Command failed, please use temp_readout.sh -i2c_find to check access"
		fi
		exit $?
	fi
	
#Read All the Temperatures in a loop
elif [ $1 = "-start" ]
then
	COM_ERR=0
	if [ $NUM_CONFS -eq 1 ]
	then
		echo "Settings taken from $GLOBAL_CONF"
	else
		echo "Global settings taken from $GLOBAL_CONF"
	fi
	
	echo ""
	sudo $INSTALL_DIR/$LIB -start $args
	exit $?

	

#Write the .conf file
elif [ $1 = "-config" ]
then
	COM_ERR=0
	echo "Please use a text editor to edit the configuration file"
	echo "The location of the configuration file is $CONF_DIR"
	echo "For example : sudo nano $CONF_DIR/temp_readout_1.conf"
	exit $?
	
#Read the device settings
#If just one board and one .conf file then temp_readout.sh -read_settings
#If more than one board, then  temp_readout.sh -read_settings <.conf file name>
elif [ $1 = "-read_settings" ]
then
	COM_ERR=0
	
	if [ $NUM_CONFS -ne 1 ] && [ $# -ne 2 ]
	then
		echo "ERROR: Expecting one argument, <.conf file name>"
		echo "For example: temp_readout.sh -read_settings temp_readout_1.conf"
		echo "Listing .conf files: ls $CONF_DIR"
		ls $CONF_DIR
		exit 1
	
	else	
	
		if [ $NUM_CONFS -ne 1 ]
		then
			
			if [ -f $CONF_DIR/$2 ]
			then
				source $CONF_DIR/$2
			else
				echo "File $CONF_DIR/$2 not found"
				exit 1;
			fi
				
		else
			source $CONF_DIR/*
		fi
		
		sudo $INSTALL_DIR/$LIB $args -read_settings $I2C_ADDR
		exit $?
	fi

	
#Scans the i2c bus to find the device
#Note: I2C address inside .conf file will be updated automatically 
elif [ $1 = "-i2c_find" ]
then
	COM_ERR=0
	sudo $INSTALL_DIR/$LIB -i2c_find
	NUM_BOARDS=$?
	
	if [ $NUM_BOARDS -eq 1 ] &&  [ $NUM_CONFS -eq 1 ]
	then
		sudo $INSTALL_DIR/$LIB -i2c_get_addr
		tmp="$(printf '%x\n' $?)"
		F=$CONF_DIR/$(ls $CONF_DIR)
		echo "Updating I2C_ADDR variable inside $F to $tmp"
		sudo sed -i "/I2C_ADDR=/c\I2C_ADDR=$tmp" $F	
	elif [ $NUM_BOARDS -ne 0 ]
	then 
		echo "Found $NUM_BOARDS boards"
		echo "Remember to setup the I2C_ADDR variable inside the .conf file(s)"
		echo "The .conf files are located in $CONF_DIR"
	else
		echo "Didn't find any boards"
	fi
	exit $?

#Change I2C address
#Note: I2C address inside .conf file will be updated automatically 
elif [ $1 = "-i2c_change_addr" ]
then
	if [ $NUM_CONFS -ne 1 ] && [ $# -ne 3 ]
	then
		echo "ERROR: Expecting two arguments, <new_address> <.conf file name>"
		echo "For example: temp_readout.sh -i2c_change_addr 80 temp_readout_1.conf"
		echo "Listing .conf files: ls $CONF_DIR"
		ls $CONF_DIR
		exit 1
	
	elif [ $NUM_CONFS -eq 1 ] && [ $# -ne 2 ]
	then
		echo "ERROR: Expecting one argument, <new_address>"
		echo "For example: temp_readout.sh -i2c_change_addr 80"
		exit 1
		
	else	
	
		if [ $NUM_CONFS -ne 1 ]
		then
			if [ -f $CONF_DIR/$3 ]
			then
				F=$CONF_DIR/$3
			else
				echo "File $CONF_DIR/$3 not found"
				exit 1;
			fi
		else
			F=$CONF_DIR/$(ls $CONF_DIR)
			
		fi
		
		source $F
		
		sudo $INSTALL_DIR/$LIB -i2c_addr $I2C_ADDR -i2c_change_addr $2 
		if [ $? -eq 0 ]
		then
			echo "Updating I2C_ADDR variable inside $F to $2"
			sudo sed -i "/I2C_ADDR=/c\I2C_ADDR=$2" $F
		else
			echo "ERROR: Failed to update I2C address."
			echo "Please use -i2c_find to find the I2C address, then verify that I2C_ADDR inside the .conf file matches"
		fi
		exit $?
	fi

#Print software revision
elif [ $1 = "-sw_rev" ]
then
	sudo $INSTALL_DIR/$LIB -sw_rev
	exit $?
	
	
#Read the .conf file
elif [ $1 = "-read_conf" ]
then
	COM_ERR=0
	
	if [ $NUM_CONFS -ne 1 ] && [ $# -ne 2 ]
	then
		echo "ERROR: Expecting an argument,  <.conf file name>"
		echo  "For example temp_readout.sh -read_conf temp_readout_1.conf"
		echo "Listing .conf files: ls $CONF_DIR"
		ls $CONF_DIR
		exit 1
	else	
		if [ $NUM_CONFS -ne 1 ]
		then
			if [ -f $CONF_DIR/$2 ]
			then
				echo "Reading $CONF_DIR/$2"
				echo ""
				cat $CONF_DIR/$2
				exit $?
			else
				echo "File $CONF_DIR/$2 not found"
				exit 1;
			fi
		else
			echo "Reading $(ls $CONF_DIR)"
			echo ""
			cat $CONF_DIR/*
		fi
	fi


#Firmware update
elif [ $1 = "-fw_update" ]
then
	COM_ERR=0
	   
	if [ $NUM_CONFS -ne 1 ] && [ $# -ne 3 ]
	then
		echo "ERROR: Expecting two arguments,  <filename> and <.conf file name>"
		echo  "For example temp_readout.sh -fw_update file.bin temp_readout_1.conf "
		echo "Listing .conf files: ls $CONF_DIR"
		ls $CONF_DIR
		exit 1
	
	else	

		if [ $NUM_CONFS -ne 1 ]
		then
			
			if [ -f $CONF_DIR/$3 ]
			then
				source $CONF_DIR/$3
			else
				echo "File $CONF_DIR/$3 not found"
				exit 1;
			fi
				
		else
			source $CONF_DIR/*
		fi
		
      if [ -f $2 ]
		then
			sudo $INSTALL_DIR/$LIB -i2c_addr $I2C_ADDR -fw_update $2
			if [ $? -ne 0 ]
			then
				echo "Command failed, please use temp_readout.sh -i2c_find to check access"
			fi 
		else
			echo "File $2 doesn't exist"	
		fi
		exit $?
	fi
	
elif [ $1 = "-ntc_calc_beta" ]
then
	
	if [ $# -ne 6 ]
	then
		echo "Number of arguments incorrect, example of valid call is temp_readout.sh -ntc_calc_beta C 25 8888.8 50 5555.5"
	else
		COM_ERR=0
		sudo $INSTALL_DIR/$LIB -ntc_calc_beta $2 $3 $4 $5 $6
	fi
	
	exit $?
	
#Initialize the hardware, must be called before using -read_sensor. Not needed before calling -start.
#If -start loop is running return error
elif [ $1 = "-init_hw" ]
then
	COM_ERR=0
	
	if pgrep -x "temp_readout.li" > /dev/null
	then
		echo "ERROR: Command cannot be run while the main program loop is running."
	else
		if [ $NUM_CONFS -eq 1 ]
		then
			echo "Settings taken from $GLOBAL_CONF"
		else
			echo "Global settings taken from $GLOBAL_CONF"
		fi
		echo ""
		sudo $INSTALL_DIR/$LIB -init_hw $args
	fi
	
	exit $?

#Turn the buzzer on/off in manual mode
elif [ $1 = "-buzzer" ]
then
	COM_ERR=0
	
	if pgrep -x "temp_readout.li" > /dev/null
	then
		echo "ERROR: Command cannot be run while the main program loop is running."
	elif [ $NUM_CONFS -ne 1 ]
	then
		if [ $# -ne 2 ]
		then
			echo "ERROR: Expecting another argument, <.conf file name>"
			echo "For example: temp_readout.sh -buzzer temp_readout_1.conf"
			echo "Listing .conf files: ls $CONF_DIR"
			ls $CONF_DIR
			exit 1
		else
		
			if [ -f $CONF_DIR/$2 ]
			then
				source $CONF_DIR/$2
				sudo $INSTALL_DIR/$LIB $args -buzzer $I2C_ADDR
				exit $?
			else
				echo "File $CONF_DIR/$2 not found"
				exit 1;
			fi
		fi
	
	else
		sudo $INSTALL_DIR/$LIB $args -buzzer $I2C_ADDR
		exit $?
	fi	
	
#Read a sensor, used to manually read sensors instead of using the -start command.
#init_hw has to be called once prior to calling read_sensor. NTC's are numbered 1 - 32 and
#thermocouples from 33 - 36.
elif [ $1 = "-read_sensor" ]
then
	COM_ERR=0
	
	if pgrep -x "temp_readout.li" > /dev/null
	then
		echo "ERROR: Command cannot be run while the main program loop is running."
	elif [ $NUM_CONFS -ne 1 ]
	then
		if [ $# -lt 3 ]
		then
			echo "ERROR: Expecting at least two arguments, <.conf file name> and <sensor_num> "
			echo "For example: temp_readout.sh -read_sensor temp_readout_1.conf 12 "
			echo "Listing .conf files: ls $CONF_DIR"
			ls $CONF_DIR
			exit 1
		else
		
			if [ -f $CONF_DIR/$2 ]
			then
				source $CONF_DIR/$2
				sudo $INSTALL_DIR/$LIB $args -read_sensor $I2C_ADDR ${@:3}
				exit $?
			else
				echo "File $CONF_DIR/$2 not found"
				exit 1;
			fi
		fi
	
	else
		if [ $# -lt 2 ]
		then
			echo "ERROR: Expecting at least one argument, <sensor_num>"
			echo "For example: temp_readout.sh -read_sensor 12"
			exit 1
		
		else	
			sudo $INSTALL_DIR/$LIB $args -read_sensor $I2C_ADDR ${@:2}
			exit $?
		fi
	fi	
fi

if [ $COM_ERR = "1" ]
then
	echo "Error: Unknown command $1, please use -help to see the list of commands"
fi
	
	